<?php
/*
Plugin Name: Ekultura Export
Description: Rozszerzenie służące do eksportu danych do portalu kulturawzasiegu.pl
Version: 1.0
Author: nFinity.pl i KPCK
License: GPL-2.0+
License URI: http://www.opensource.org/licenses/gpl-license.php
*/

if (!defined('ABSPATH')) exit;

if (!class_exists('Ekultura_Export')) {

	$api_url = 'http://kulturawzasiegu.pl/wp-api/';
	$plugin_slug = basename(dirname(__FILE__));

	function ekultura_prepare_request($action, $args)
	{
		global $wp_version;

		return array(
			'body' => array(
				'action' => $action,
				'request' => serialize($args),
				'api-key' => md5(get_bloginfo('url'))
			),
			'user-agent' => 'WordPress/' . $wp_version . '; ' . get_bloginfo('url')
		);
	}

	add_filter('pre_set_site_transient_update_plugins', function ($checked_data) {
		global $api_url, $plugin_slug;

		if (empty($checked_data->checked)) {
			return $checked_data;
		}

		$request_args = array(
			'slug' => $plugin_slug,
			'version' => $checked_data->checked[$plugin_slug . '/' . $plugin_slug . '.php'],
		);

		$request_string = ekultura_prepare_request('basic_check', $request_args);

		$raw_response = wp_remote_post($api_url, $request_string);

		if (!is_wp_error($raw_response) && ($raw_response['response']['code'] == 200)) {
			$response = unserialize($raw_response['body']);
		}

		if (
			isset($response) && is_object($response) && !empty($response)
			&& version_compare($request_args['version'], $response->version, '<')
		) {
			$response->new_version = $response->version;
			$checked_data->response[$plugin_slug . '/' . $plugin_slug . '.php'] = $response;
		}


		return $checked_data;
	});


	add_filter('plugins_api', function ($def, $action, $args) {
		global $plugin_slug, $api_url;

		if (!isset($args->slug) or $args->slug != $plugin_slug)
			return $def;

		$plugin_info = get_site_transient('update_plugins');
		$current_version = $plugin_info->checked[$plugin_slug . '/' . $plugin_slug . '.php'];
		$args->version = $current_version;

		$request_string = ekultura_prepare_request($action, $args);

		$request = wp_remote_post($api_url, $request_string);

		if (is_wp_error($request)) {
			$res = new WP_Error('plugins_api_failed', __('An Unexpected HTTP Error occurred during the API request.</p> <p><a href="?" onclick="document.location.reload(); return false;">Try again</a>'), $request->get_error_message());
		} else {
			$res = unserialize($request['body']);

			if ($res === false)
				$res = new WP_Error('plugins_api_failed', __('An unknown error occurred'), $request['body']);
		}


		return $res;
	}, 11, 3);




	class Ekultura_Export
	{
		public static $prefix = '';

		public function __construct()
		{
			if (empty(Ekultura_Export::$prefix)) {
				Ekultura_Export::$prefix = md5(get_site_url()) . '_';
			}

			add_action('plugins_loaded', array($this, 'constants'));
			add_action('plugins_loaded', array($this, 'includes'));
			add_action('init', array($this, 'feed'));
		}

		public function constants()
		{
			if (!defined('EKULTURA_EXPORT_DIR'))
				define('EKULTURA_EXPORT_DIR', trailingslashit(plugin_dir_path(__FILE__)));

			if (!defined('EKULTURA_EXPORT_URL'))
				define('EKULTURA_EXPORT_URL', trailingslashit(plugin_dir_url(__FILE__)));

			if (!defined('EKULTURA_EXPORT_VERSION'))
				define('EKULTURA_EXPORT_VERSION', '1.0');

			if (!defined('EKULTURA_EXPORT_INCLUDES'))
				define('EKULTURA_EXPORT_INCLUDES', EKULTURA_EXPORT_DIR . trailingslashit('includes'));
		}

		public function includes()
		{
			require_once(EKULTURA_EXPORT_INCLUDES . 'register-custom-fields.php');
		}


		public function feed()
		{
			if (isset($_GET['ekultura_action'])) {
				// $time_start = microtime(true);
				wp_deregister_script('jquery');
				wp_deregister_script('jquery-ui');
				remove_action('wp_footer', 'wpa_jquery_asl', 100);

				$xml = new SimpleXMLElement('<feeds/>');
				$xml->addAttribute('version', '2');
				$xml->addAttribute('pver', EKULTURA_EXPORT_VERSION);

				$query = new WP_Query(array(
					'post_type' => array('event', 'post', 'exhibitions', 'tribe_events', 'ajde_events'),
					'posts_per_page' => -1,
					'date_query' => array(
						array(
							'column' => 'post_modified_gmt',
							'after'  => '7 day ago',
						),
					),
					'meta_query' => array(
						array(
							'key' => 'export_publish',
							'value' => '1',
						)
					)
				));


				$time_limit = time() - 86400 * 7;
				foreach ($query->get_posts() as $post) {

					// dodatakowe odrzucanie postow modyfikowanych < 7 dni
					if (strtotime($post->post_modified_gmt) < $time_limit) continue;
					$meta = get_post_meta($post->ID);

					$organizerName = isset($meta['organizer_post']) ? $meta['organizer_post'][0] : '';
					$cityName = isset($meta['export_city']) ? $meta['export_city'][0] : '';

					$placeName = isset($meta['organizer_object_post']) ? $meta['organizer_object_post'][0] : '';
					$isFree = isset($meta['export_is_free']) ? $meta['export_is_free'][0] : '';
					$evType = isset($meta['export_type']) ? $meta['export_type'][0] : 0;
					$isExhibition = isset($meta['export_is_exhibition']) ? $meta['export_is_exhibition'][0] : '';


					if (isset($meta['_EventStartDate'])) {
						if (isset($meta['_EventOrganizerID'][0])) {
							$organizer = get_post($meta['_EventOrganizerID'][0]);
							if ($organizer) {
								$organizerName = $organizer->post_title;
							}
						}

						if (isset($meta['_EventVenueID'][0])) {

							$place = get_post($meta['_EventVenueID'][0]);
							if ($place) {
								$placeMeta = get_post_meta($meta['_EventVenueID'][0]);

								$cityName = $placeMeta['_VenueCity'][0] . ', ' . $placeMeta['_VenueAddress'][0];
								$placeName = isset($placeMeta['_VenueVenue'][0]) ? $placeMeta['_VenueVenue'][0] : '';
							}
						}

						if (isset($meta['_EventCost']) && $meta['_EventCost'][0] == 0) {
							$isFree = 1;
						} else {
							$isFree = 0;
						}
					}

					if (isset($meta['evcal_organizer'])) {
						$organizerName = isset($meta['evcal_organizer']) ? $meta['evcal_organizer'][0] : '';
						$placeName = isset($meta['evcal_location_name']) ? $meta['evcal_location_name'][0] : '';
						$cityName = isset($meta['evcal_location']) ? $meta['evcal_location'][0] : '';
					}

					$xmlFeed = $xml->addChild('feed');
					$xmlFeed->type = 5;
					$xmlFeed->url = get_permalink($post);
					$xmlFeed->title = $post->post_title;
					$xmlFeed->short = $post->post_excerpt;
					$xmlFeed->body = strip_shortcodes($post->post_content);
					$xmlFeed->city = $cityName;
					$xmlFeed->for_kids = $meta['export_for_kids'][0];
					$xmlFeed->event_is_free = $isFree;
					$xmlFeed->event_is_exhibition = $isExhibition;
					$xmlFeed->event_type = $evType;
					$xmlFeed->organizer = $organizerName;
					$xmlFeed->event_place = $placeName;
					$xmlFeed->category = isset($meta['export_category']) ? $meta['export_category'][0] : '';
					$xmlFeed->id = self::$prefix . $post->ID;
					$thumb = wp_get_attachment_image_src(get_post_thumbnail_id($post->ID), 'large');

					if ($thumb !== false) $xmlFeed->thumb_url = $thumb[0];

					$startArray = array();
					$endArray = array();

					if (isset($meta['evcal_srow'])) {
						$xmlFeed->type = 1;

						$startDate = new DateTime(date('c', $meta['evcal_srow'][0]));
						if (isset($meta['evcal_allday']) && $meta['evcal_allday'][0] == 'yes') {
							$startDate->setTime(0, 0, 0);
						}
						$startArray[] = $startDate->format('d.m.Y H:i');

						$endDate = new DateTime(date('c', $meta['evcal_erow'][0]));
						if (isset($meta['evcal_allday']) && $meta['evcal_allday'][0] == 'yes') {
							$endDate->setTime(23, 59, 59);
						}
						$endArray[] = $endDate->format('d.m.Y H:i');

						$repeats = @unserialize($meta['repeat_intervals'][0]);
						if (is_array($repeats) && count($repeats) > 0 && isset($meta['evcal_repeat']) && $meta['evcal_repeat'][0] == 'yes') {
							$startArray = [];
							$endArray = [];

							foreach ($repeats as $singleRepeat) {
								$startDate = new DateTime(date('c', $singleRepeat[0]));
								if (isset($meta['evcal_allday']) && $meta['evcal_allday'][0] == 'yes') {
									$startDate->setTime(0, 0, 0);
								}
								$startArray[] = $startDate->format('d.m.Y H:i');

								$endDate = new DateTime(date('c', $singleRepeat[1]));
								if (isset($meta['evcal_allday']) && $meta['evcal_allday'][0] == 'yes') {
									$endDate->setTime(23, 59, 59);
								}
								$endArray[] = $endDate->format('d.m.Y H:i');
							}
						}
					} else
					if (isset($meta['_EventStartDate'])) {
						$xmlFeed->type = 1;
						$startArray[] = DateTime::createFromFormat('Y-m-d H:i:s', $meta['_EventStartDateUTC'][0])->format('d.m.Y H:i');
						$endArray[] = DateTime::createFromFormat('Y-m-d H:i:s', $meta['_EventEndDateUTC'][0])->format('d.m.Y H:i');
					} else
					if (isset($meta['fc_start_datetime'])) {
						$xmlFeed->type = 1;
						for ($i = 0; $i < count($meta['fc_start_datetime']); $i++) {

							$start = DateTime::createFromFormat('Y-m-d H:i:s', $meta['fc_start_datetime'][$i]);
							if ($start) {
								$startArray[] = $start->format('d.m.Y H:i');
							}

							$end = DateTime::createFromFormat('Y-m-d H:i:s', $meta['fc_end_datetime'][$i]);
							if ($end) {
								$endArray[] = $end->format('d.m.Y H:i');
							}
						}
					} else
					if (isset($meta['organizer_object_dates'])) {
						$dates = explode("\n", $meta['organizer_object_dates'][0]);
						if (count($dates)) {
							$xmlFeed->type = 1;
						}
						foreach ($dates as $date) {
							$date = explode('-', $date);

							$dateParse = DateTime::createFromFormat('d.m.Y H:i', trim($date[0]));
							if (!$dateParse) {
								$dateParse = DateTime::createFromFormat('d.m.Y H:i', trim($date[0]) . ' 00:00');
							}
							if ($dateParse) {
								$startArray[] = $dateParse->format('d.m.Y H:i');
							}

							if (isset($date[1])) {
								$dateParse = DateTime::createFromFormat('d.m.Y H:i', trim($date[1]));
								if (!$dateParse) {
									$dateParse = DateTime::createFromFormat('d.m.Y H:i', trim($date[1]) . ' 23:59');
								}
								if ($dateParse) {
									$endArray[] = $dateParse->format('d.m.Y H:i');
								}
							} else {
								if ($dateParse) {
									$endArray[] = $dateParse->format('d.m.Y 23:59');
								}
							}
						}
					} else {
						global $wpdb;
						$additionalDates = $wpdb->get_results("SELECT * FROM calendar WHERE post_id = " . $post->ID);

						if (count($additionalDates) > 0) {
							$xmlFeed->type = 1;
						}

						for ($i = 0; $i < count($additionalDates); $i++) {

							$start = new DateTime($additionalDates[$i]->startdate);
							if ($start) {
								$startArray[] = $start->format('d.m.Y H:i');
							}

							$end = new DateTime($additionalDates[$i]->enddate);
							if ($end) {
								$endArray[] = $end->format('d.m.Y H:i');
							}
						}
					}

					$fcFields = array();
					foreach ($meta as $key => $val) {
						if (substr($key, 0, 3) == 'fc_') {
							$fcFields[$key] = $val;
						}
					}

					$xmlFeed->event_fc = base64_encode(serialize($fcFields));
					$xmlFeed->event_date_from = implode(',', $startArray);
					$xmlFeed->event_date_to = implode(',', $endArray);
					$xmlFeed->photos = '';

					$ut = get_the_time('U', $post->ID);

					$xmlFeed->created_at = $ut;
					$xmlFeed->published_from = $ut;
					$xmlFeed->modified_at = get_post_modified_time('U', true, $post->ID);
				}
				// $time_end = microtime(true);

				// echo ($time_end - $time_start);
				header('Content-type: text/xml');
				echo $xml->asXML();
				exit;
			}
		}
	}
}

$Ekultura_Export = new Ekultura_Export();
